# Changelog

Alle relevanten Änderungen am FIT-Connect Java SDK werden in dieser Datei dokumentiert.

Das Format basiert auf [Keep a Changelog](https://keepachangelog.com/en/)
und hält sich an [Semantic Versioning](https://semver.org/).

## [2.0.1] - 2024-03-15

### Behobene Bugs

- Die Validierung von ARS Schlüsseln bei der Abfrage von Routing-Informationen wurde so angepasst, dass auch Regionen die in einer anderen Region enthalten sind, korrekt geprüft werden (planning#1684)

### Änderungen
Um im Log keine sensiblen Daten zu loggen wurden folgende Anpassungen in der Logback Konfiguration vorgenommen:
- Das Loglevel für den Schema Validator wurde für `com.networknt.*` auf INFO gesetzt
- Das Loglevel für den OkHttp Client wurde für ``okhttp3.*`` auf INFO gesetzt
- 
### Aktualisierte Bibliotheken

- JWK-Validator 2.1.0
- Jackson 2.17.0
- Logback 1.5.3
- Mockito 5.11
- Maven GPG Plugin 3.2.0
- Maven Assembly Plugin 3.7.0
- Maven Git Commit ID Plugin 8.0.1

## [2.0.0] - 2024-02-20

### Hinzugefügt
- Bidirektionale Kommunikation mit FIT-Connect Antwortkanal (planning#654)
  - neue Submissions für einen existierenden Case senden
  - Antworten (Replies) auf Submissions senden
  - Antworten empfangen
  - Liste der verfügbaren Cases und Replies
  - aktuellen Status für Replies abfragen
  - Neuer ``FitConnect``-ReplyChannel, der den öffentlichen Reply-Verschlüsselungsschlüssel und eine Liste von Verarbeitungsstandards überträgt
  - `ReplyChannelKeyGenerator` zur Erzeugung von ephemeral-keys für die Ver- und Entschlüsselung von Replies
- ServiceType-Getter an der ReceivedSubmission (planning#1072)
- Setzen des ApplicationDate an der SendableSubmission über `setApplicationDate(localDate)` (planning#1243)
  - Zurückdatieren des Einreichungsdatums
  - unabhängig vom technischen issued-at Zeitstempel der Submission
- Konfiguration von HTTP-Client-Timeouts in der neuen HttpConfig (planning#1664)
  - readTimeout setzen
  - writeTimeout setzen
  - connectionTimeout setzen
- Für die Validierung lokaler Fachdatenschemata können beliebige Schema-formate vom lokalen Dateisystem über die `config.yaml` oder `ApplicationConfig` hinzugefügt werden (planning#1196)
- Neue DTOs für Callback-Benachrichtigung über neue Submissions und Replies (planning#1400)
- Zugriff auf den technischen submitted-at (iat) Zeitstempel an der `ReceivedSubmission`  (planning#654)
- Neuer ``KeyGenerator`` zur Erzeugung von X.509-Testzertifikaten, RSA-KeyPairs und JWKs (planning#664)

### Änderungen
> **Details zu den Änderungen im Zusammenhang mit diesem Update entnehmen Sie bitte dem** [Update Guide](https://git.fitko.de/fit-connect/sdk-java/-/blob/main/docs/update-guide.md)

- Aktualisierte Schemata:
  - Metadaten-Schema 1.2.0
  - SET-Schema 1.2.1
- Model-Klassen vom Client-Modul wurden in das API-Modul verschoben (planning#654)
- Umbenennung von API-Methoden (planning#654):
  - statische ClientFactory-Methoden zu ``ClientFactory.create...(config)``
  - statische ReplyChannel-Methoden zu ``ReplyChannel.of...()``
  - ``DefaultEnvironments`` für TEST, STAGE und PROD wurden umbenannt in `Environments`
- Das EventLog Status Objekt wurde umbenannt von ``SubmissionStatus`` in `Status` (planning#654)
  - dieses wird sowohl für ``getSubmissionStatus`` als auch für ``getReplyStatus`` verwendet
  - enthält das neue ``issuedAt`` Datum
- Die ``ReceivedSubmission`` implementiert ``Comparable<Date>`` und wird autom. nach `issuedAt` sortiert (planning#654)
- Neues ``HttpConfig``-Objekt (planning#1664):
  - enthält HTTP `TimeoutConfig`
  - `ProxyConfig` wurde nach `HttpConfig` verschoben
- Die Polling-Methode ``getAvailableSubmissionsForDestination`` (ohne Limit und Offset) lädt nun implizit ALLE verfügbaren Einreichungen (rekursiv in 25er-Chunks). Die macht es einfacher, eine Liste aller verfügbaren Einreichungen zu laden, ohne explizit Paging einzusetzen. (Planung#654)
- Der JWKValidator 2.0.0 cached Anfragen, was die Geschwindigkeit der Schlüsselüberprüfung verbessert
- Detailliertere Protokollierung von Requests mit HTTP-Methode, Statuscode und benötigter Zeit (planning#654)

### Behobene Bugs

- Das Zeitstempelformat in der Callback-Validierung wurde von Millisekunden auf Sekunden korrigiert (planning#1400)
- Korrigierte Json Validator Locale, welche zu Build-Fehlern in Umgebungen mit deutscher Locale führte (planning#1370)
- Der korrekte Content-Type Header (cty) wurde zum JWE-Header hinzugefügt (planning#1244)
- Ein Fehler, der bei der Validierung von Routing-Services mit mehreren Diensten innerhalb der JWT-Claims auftrat, wurde behoben (planning#1481)

### Veraltet
Die Attachment ``fromString``-Methoden werden ab SDK-Version `3.0.0` als deprecated markiert, da sie bei falscher Verwendung zu Problemen mit der Zeichencodierung führen können (planning#1308).

### Entfernt
- Command-Line-Client (planning#1571)
  - der ausführbare Kommandozeilen-Client wurde aus dem SDK entfernt und ist nun ein eigenständiges Projekt
  - siehe [FIT-Connect Tools Repository](https://git.fitko.de/fit-connect/fit-connect-tools/) für weitere Informationen
- Apache Tika-Library (wurde vom cli-Modul verwendet)
- JCommander-Library (wurde vom cli-Modul verwendet)

### Aktualisierte Bibliotheken

- JWK-Validator 2.0.0
- Nimbus/Jose 9.37.3
- OkHttp 5.0.0-alpha.12
- Lombok 1.18.30
- Jackson 2.16.1
- Logback 1.5.0
- SLF4J 2.0.12
- SnakeYaml 2.2
- JSON Schema Validator 1.3.3
- Junit 5.10.2
- Mockito 5.10
- Wiremock 3.0.1

## [1.0.0] - 2023-08-04

### Hinzugefügt
- JWK Test-Zertifikat Generator
- Integration des Demo-Projekts ins SDK

### Änderungen
- JWK-Validator 1.5.1
- Mockito Core 5.4.0
- JSON Schema Validator 1.0.86
- Maven 3.9.4
- OpenCSV 5.8

## [1.0.0-rc.1] - 2023-03-05

- Initialer Release-Candidate des SDKs.
