package dev.fitko.fitconnect.api.domain.model.event.authtags;

import com.nimbusds.jose.JWEObject;
import dev.fitko.fitconnect.api.exceptions.internal.AuthenticationTagException;
import lombok.Data;

import java.text.ParseException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Data
public class AuthenticationTags {

    private String metadata;
    private String data;
    private Map<UUID, String> attachments;

    public static AuthenticationTags from(final String encryptedData, final String encryptedMetadata, final Map<UUID, String> encryptedAttachments) {
        final var authenticationTags = new AuthenticationTags();
        authenticationTags.setData(fromData(encryptedData));
        authenticationTags.setMetadata(fromMetadata(encryptedMetadata));
        authenticationTags.setAttachments(buildAttachmentAuthenticationTags(encryptedAttachments));
        return authenticationTags;
    }

    public static String fromData(final String encryptedData) {
        return buildAuthTag(encryptedData);
    }

    public static String fromMetadata(final String encryptedMetadata) {
        return buildAuthTag(encryptedMetadata);
    }

    public static Map<UUID, String> fromAttachments(final Map<UUID, String> encryptedAttachments) {
        return buildAttachmentAuthenticationTags(encryptedAttachments);
    }

    private static Map<UUID, String> buildAttachmentAuthenticationTags(final Map<UUID, String> encryptedAttachments) {
        if (encryptedAttachments == null || encryptedAttachments.isEmpty()) {
            return Collections.emptyMap();
        }
        final var authTags = new HashMap<UUID, String>();
        encryptedAttachments.forEach((key, value) -> authTags.put(key, buildAuthTag(value)));
        return authTags;
    }

    private static String buildAuthTag(final String base64EncodedJWT) throws AuthenticationTagException {
        if (base64EncodedJWT == null) {
            return null;
        }
        try {
            return JWEObject.parse(base64EncodedJWT).getAuthTag().toString();
        } catch (final ParseException e) {
            throw new AuthenticationTagException(e.getMessage(), e);
        }
    }
}
