package dev.fitko.fitconnect.core.auth;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import dev.fitko.fitconnect.api.domain.auth.OAuthToken;
import dev.fitko.fitconnect.api.services.auth.OAuthService;
import dev.fitko.fitconnect.core.RestEndpointBase;
import dev.fitko.fitconnect.core.http.HttpClient;
import dev.fitko.fitconnect.core.http.MimeTypes;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static com.github.tomakehurst.wiremock.client.WireMock.ok;
import static com.github.tomakehurst.wiremock.client.WireMock.post;
import static com.github.tomakehurst.wiremock.client.WireMock.urlEqualTo;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;

class DefaultOAuthApiServiceTest extends RestEndpointBase {

    private OAuthService underTest;

    @BeforeEach
    public void setUp() {
        wireMockServer.resetMappings();
        final var config = getTestConfig("http://localhost:" +  wireMockServer.port());
        underTest = new DefaultOAuthApiService(new HttpClient(), "id", "secret", config.getAuthBaseUrl());
    }

    @Test
    void getCurrentToken() throws JsonProcessingException {
        // Given
        final OAuthToken token= new OAuthToken();
        token.setAccessToken("123");
        token.setExpiresIn(30);

        stubForTokenPostRequest(token);

        // When
        final OAuthToken currentToken = underTest.getCurrentToken();

        // Then
        assertThat(currentToken.getExpiresIn(), is(30));
        assertThat(currentToken.getAccessToken(), is("123"));
    }

    @Test
    void refreshTokenWhenExpired() throws JsonProcessingException {
        // Given
        final OAuthToken oldToken = new OAuthToken();
        oldToken.setAccessToken("123");
        oldToken.setExpiresIn(0);

        // Set expired token so the next call needs to refresh
        stubForTokenPostRequest(oldToken);

        // Refresh
        wireMockServer.resetMappings();
        final OAuthToken refreshed = new OAuthToken();
        refreshed.setAccessToken("456");
        refreshed.setExpiresIn(1800);

        // When
        stubForTokenPostRequest(refreshed);
        final OAuthToken refreshedToken = underTest.getCurrentToken();

        // Then
        assertThat(refreshedToken.getExpiresIn(), is(1800));
        assertThat(refreshedToken.getAccessToken(), is("456"));
    }

    private void stubForTokenPostRequest(final OAuthToken token) throws JsonProcessingException {
        wireMockServer.stubFor(
                post(urlEqualTo("/token"))
                        .willReturn(ok()
                                .withBody(new ObjectMapper().writeValueAsString(token))
                                .withHeader("Content-Type", MimeTypes.APPLICATION_JSON)
                                .withStatus(200)));
    }
}