package dev.fitko.fitconnect.sdk.examples;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.config.SenderConfig;
import dev.fitko.fitconnect.api.config.defaults.Environments;
import dev.fitko.fitconnect.api.domain.model.callback.CallbackHeaderKeys;
import dev.fitko.fitconnect.api.domain.model.callback.NewSubmissionsCallback;
import dev.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import dev.fitko.fitconnect.api.domain.validation.ValidationResult;
import dev.fitko.fitconnect.client.SenderClient;
import dev.fitko.fitconnect.client.bootstrap.ClientFactory;
import dev.fitko.fitconnect.core.crypto.HashService;
import okhttp3.Headers;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Date;
import java.util.List;
import java.util.UUID;

public class CallbackValidationExample {

    private static final Logger LOGGER = LoggerFactory.getLogger(CallbackValidationExample.class);

    public static void main(String[] args) throws JsonProcessingException {

        // expected HTTP-Response body and headers from the callback you received
        var httpResponseBody = getHttpResponseBody();
        var httpResponseHeaders = getHttpResponseHeaders(httpResponseBody, "callbackSecret");

        final SenderClient senderClient = getSenderClient();

        var secret = "callbackSecret";
        final String hmac = httpResponseHeaders.get(CallbackHeaderKeys.AUTHENTICATION);
        final long timestampInSec = Long.parseLong(httpResponseHeaders.get(CallbackHeaderKeys.TIMESTAMP));

        final ValidationResult validationResult = senderClient.validateCallback(hmac, timestampInSec, httpResponseBody, secret);
        LOGGER.info("Callback validation is {}", validationResult.isValid());
    }

    private static SenderClient getSenderClient() {
        final ApplicationConfig config = ApplicationConfig.builder()
                .senderConfig(new SenderConfig())
                .activeEnvironment(Environments.TEST.getEnvironmentName())
                .build();
        return ClientFactory.createSenderClient(config);
    }

    private static Headers getHttpResponseHeaders(String httpResponseBody, String callbackSecret) {
        final String calculatedHMAC = new HashService().calculateHMAC(httpResponseBody, callbackSecret);
        return Headers.of(CallbackHeaderKeys.TIMESTAMP, String.valueOf(new Date().getTime()), CallbackHeaderKeys.AUTHENTICATION, calculatedHMAC);
    }

    private static String getHttpResponseBody() throws JsonProcessingException {
        var type = "https://schema.fitko.de/fit-connect/submission-api/callbacks/new-submissions";
        var newSubmissions = List.of(new SubmissionForPickup(UUID.randomUUID(), UUID.randomUUID(), UUID.randomUUID()));
        return new ObjectMapper().writeValueAsString(new NewSubmissionsCallback(type, newSubmissions));
    }
}
