package dev.fitko.fitconnect.sdk.examples;

import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.domain.model.attachment.Attachment;
import dev.fitko.fitconnect.api.domain.model.event.Status;
import dev.fitko.fitconnect.api.domain.model.submission.SentSubmission;
import dev.fitko.fitconnect.api.domain.sender.SendableEncryptedSubmission;
import dev.fitko.fitconnect.api.domain.sender.SendableSubmission;
import dev.fitko.fitconnect.client.SenderClient;
import dev.fitko.fitconnect.client.bootstrap.ApplicationConfigLoader;
import dev.fitko.fitconnect.client.bootstrap.ClientFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URI;
import java.nio.file.Path;
import java.util.Objects;
import java.util.UUID;

public class SenderExample {

    private static final Logger LOGGER = LoggerFactory.getLogger(SenderExample.class);

    /**
     * Fill out config.yml in /resources and provide a DESTINATION_ID that matches the configuration (serviceType, data schema, etc.)
     */
    public static void main(String[] args) {

        final ApplicationConfig config = ApplicationConfigLoader.loadConfigFromPath(Path.of("examples/src/main/resources/config.yml"));
        final SenderClient senderClient = ClientFactory.createSenderClient(config);

        sendSubmission(senderClient);
        sendAlreadyEncryptedSubmission(senderClient);
    }

    private static void sendSubmission(SenderClient senderClient) {

        final SendableSubmission submission = SendableSubmission.Builder()
                .setDestination(UUID.fromString("YOUR_DESTINATION_ID"))
                .setServiceType("urn:de:fim:leika:leistung:99400048079000", "Test-Service")
                .setJsonData(getDataAsString("/data.json"), URI.create("https://schema.fitko.de/fim/s00000114_1.1.schema.json"))
                .addAttachment(Attachment.fromPath(Path.of("examples/src/main/resources/attachment.txt"), "application/text"))
                .build();

        final SentSubmission sentSubmission = senderClient.send(submission);
        final Status submissionStatus = senderClient.getSubmissionStatus(sentSubmission);

        LOGGER.info("Current status for submission {}: {}", sentSubmission.getSubmissionId(), submissionStatus.getState());
    }

    private static void sendAlreadyEncryptedSubmission(SenderClient senderClient) {

        final SendableEncryptedSubmission submission = SendableEncryptedSubmission.Builder()
                .setDestination(UUID.fromString("YOUR_DESTINATION_ID"))
                .setServiceType("urn:de:fim:leika:leistung:99400048079000", "Test-Service")
                .setEncryptedMetadata("€ncrypt€dM€t@d@t@")
                .setEncryptedData("€ncrypt€dd@t@")
                .addEncryptedAttachment(UUID.randomUUID(), "€ncrypt€d@tt@chm€nt")
                .build();

        final SentSubmission sentSubmission = senderClient.send(submission);
        final Status submissionStatus = senderClient.getSubmissionStatus(sentSubmission);

        LOGGER.info("Current status for submission {}: {}", sentSubmission.getSubmissionId(), submissionStatus.getState());
    }

    private static String getDataAsString(final String path) {
        try {
            return new String(Objects.requireNonNull(SenderExample.class.getResourceAsStream(path)).readAllBytes());
        } catch (final IOException e) {
            throw new RuntimeException(e.getMessage(), e);
        }
    }
}
